@@ -21,6 +21,8 @@ void deltaFn();
 */
 //extern "C"{
    void ibsPrc(double* prA,int* nr,int* nc,double* ibs){
+         signal(SIGINT, &userInt);
+
          int nn;
          nn = (*nr)*((*nr)+1)/2;
          double* ptr[(*nr)*3]; for(int i=0; i<(*nr)*3; i++) ptr[i]=prA+i*(*nc);
@@ -28,23 +30,30 @@ void deltaFn();
          double* ibsp[nn]; for(int i=0; i<nn; i++) ibsp[i]=ibs+i*9;
 
          ibsPr(prAp,(*nr),(*nc),ibsp);
+         if(stopIt) {stopIt = 0; error(_("Exit without finish.\a\n"));}
    }
 
    void ibsFnc(int* gdat,int* nr,int* nc,double* ibs){
+         signal(SIGINT, &userInt);
+
          int nn;
          nn = (*nr)*((*nr)+1)/2;
          int* gdatp[(*nr)]; for(int i=0;i<(*nr);i++) gdatp[i]=gdat+i*(*nc);
          double* ibsp[nn]; for(int i=0;i<nn;i++) ibsp[i]=ibs+i*9;
 
          ibsFn(gdatp,(*nr),(*nc),ibsp);
+         if(stopIt) {stopIt = 0; error(_("Exit without finish.\a\n"));}
    }
 
    void deltaFnc(int* gdat,int* nr,int* nc,double* delta){
+         signal(SIGINT, &userInt);
+
          int nn;
          nn = (*nr)*((*nr)+1)/2;
          int* gdatp[(*nr)]; for(int i=0;i<(*nr);i++) gdatp[i]=gdat+i*(*nc);
          double* deltap[nn]; for(int i=0;i<nn;i++) deltap[i]=delta+i*5;
          deltaFn(gdatp,(*nr),(*nc),deltap);
+         if(stopIt) {stopIt = 0; error(_("Exit without finish.\a\n"));}
    }
 //}
 
@@ -80,6 +89,7 @@ void ibsPr(double*** prA,int nr,int nc,double** ibs){
    double P2[3][3];
    for(int i=0; i<3; i++){
       for(int j=0; j<3; j++){
+         if(stopIt) return;
          P2[i][j] = Pa[i]*Pa[j]
                  + (1-Pa[i])*(1-Pa[j]);
       }
@@ -88,6 +98,7 @@ void ibsPr(double*** prA,int nr,int nc,double** ibs){
    for(int i=0; i<3; i++){
       for(int j=0; j<3; j++){
          for(int k=0; k<3; k++){
+            if(stopIt) return;
             P3[i][j][k] = Pa[i]*Pa[j]*Pa[k]
                        + (1-Pa[i])*(1-Pa[j])*(1-Pa[k]);
          }
@@ -98,6 +109,7 @@ void ibsPr(double*** prA,int nr,int nc,double** ibs){
       for(int j=0; j<3; j++){
          for(int k=0; k<3; k++){
             for(int l=0; l<3; l++){
+               if(stopIt) return;
                P4[i][j][k][l] = Pa[i]*Pa[j]*Pa[k]*Pa[l]
                              + (1-Pa[i])*(1-Pa[j])*(1-Pa[k])*(1-Pa[l]);
             }
@@ -109,6 +121,7 @@ void ibsPr(double*** prA,int nr,int nc,double** ibs){
       for(int j=0; j<3; j++){
          for(int k=0; k<3; k++){
             for(int l=0; l<3; l++){
+               if(stopIt) return;
                P22[i][j][k][l] = (Pa[i]*Pa[j] + (1-Pa[i])*(1-Pa[j]))
                                 *(Pa[k]*Pa[l] + (1-Pa[k])*(1-Pa[l]));
             }
@@ -124,6 +137,7 @@ void ibsPr(double*** prA,int nr,int nc,double** ibs){
       for(int b=0;b<=a;b++){
          for(int t=0; t<9; t++) ibs[ii][t] = 0.0;
          for(int k=0;k<nc;k++){
+            if(stopIt) return;
             ibs_Pr(P2, P3, P4, P22, a, b, prA, k, ibs, ii);
          }
          for(int t=0; t<9; t++) ibs[ii][t] /= nc;
@@ -144,8 +158,7 @@ double pr(int *x,int k,int o){
    }else if(o==2) for(int i=0;i<k;i++){
       s *= (x[i]-1.0)/2; //both B alleles
    }else{
-      Rprintf("o in pr: 1 or 2 only.\n");
-      exit(0);
+      error(_("o in pr: 1 or 2 only.\n"));
    }
 
    return s;
@@ -215,6 +228,7 @@ void ibsFn(int** gdat,int nr,int nc,double** ibs){
       for(int b=0;b<=a;b++){
          for(int t=0; t<9; t++) s[t] = 0.0;
          for(int j=0; j<nc; j++){
+            if(stopIt) return;
             aa = 2.0*phi_2(a,a,gdat,j);
             bb = 2.0*phi_2(b,b,gdat,j);
             ab = 4.0*phi_2(a,b,gdat,j);
@@ -293,6 +307,7 @@ void deltaFn(int** gdat,int nr,int nc,double** delta){
       for(int b=a;b<nr;b++){
          for(int t=0; t<5; t++) s[t] = 0.0;
          for(int j=0; j<nc; j++){
+            if(stopIt) return;
             s[0] += _phi_2_(a,b,gdat,j);
             s[1] += dlt1(a,b,gdat,j);
             s[2] += dlt2(a,b,gdat,j);
